@@ -24,7 +24,7 @@
  * GTK+ at ftp://ftp.gtk.org/pub/gtk/.
  */
 
-#include <config.h>
+#include "config.h"
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -82,9 +82,10 @@ struct _GdkKeymapX11
   GdkModifierType group_switch_mask;
   GdkModifierType num_lock_mask;
   GdkModifierType modmap[8];
-  gboolean sun_keypad;
   PangoDirection current_direction;
-  gboolean have_direction;
+  guint sun_keypad      : 1;
+  guint have_direction  : 1;
+  guint caps_lock_state : 1;
   guint current_serial;
 
 #ifdef HAVE_XKB
@@ -584,74 +585,114 @@ get_direction (XkbDescRec *xkb,
     return PANGO_DIRECTION_LTR;
 }
 
-static void
+static PangoDirection
+get_direction_from_cache (GdkKeymapX11 *keymap_x11,
+			  XkbDescPtr xkb,
+			  gint group)
+{
+  Atom group_atom = xkb->names->groups[group];
+
+  gboolean cache_hit = FALSE;
+  DirectionCacheEntry *cache = keymap_x11->group_direction_cache;
+
+  PangoDirection direction = PANGO_DIRECTION_NEUTRAL;
+  gint i;
+
+  if (keymap_x11->have_direction)
+    {
+      /* lookup in cache */
+      for (i = 0; i < G_N_ELEMENTS (keymap_x11->group_direction_cache); i++)
+      {
+	if (cache[i].group_atom == group_atom)
+	  {
+	    cache_hit = TRUE;
+	    cache[i].serial = keymap_x11->current_cache_serial++; /* freshen */
+	    direction = cache[i].direction;
+	    group_atom = cache[i].group_atom;
+	    break;
+	  }
+      }
+    }
+  else
+    {
+      /* initialize cache */
+      for (i = 0; i < G_N_ELEMENTS (keymap_x11->group_direction_cache); i++)
+	{
+	  cache[i].group_atom = 0;
+	  cache[i].serial = keymap_x11->current_cache_serial;
+	}
+      keymap_x11->current_cache_serial++;
+    }
+
+  /* insert in cache */
+  if (!cache_hit)
+    {
+      gint oldest = 0;
+
+      direction = get_direction (xkb, group);
+
+      /* remove the oldest entry */
+      for (i = 0; i < G_N_ELEMENTS (keymap_x11->group_direction_cache); i++)
+	{
+	  if (cache[i].serial < cache[oldest].serial)
+	    oldest = i;
+	}
+      
+      cache[oldest].group_atom = group_atom;
+      cache[oldest].direction = direction;
+      cache[oldest].serial = keymap_x11->current_cache_serial++;
+    }
+
+  return direction;
+}
+
+static int
+get_num_groups (GdkKeymap *keymap,
+		XkbDescPtr xkb)
+{
+      Display *display = KEYMAP_XDISPLAY (keymap);
+      XkbGetControls(display, XkbSlowKeysMask, xkb);
+      XkbGetUpdatedMap (display, XkbKeySymsMask | XkbKeyTypesMask |
+			XkbModifierMapMask | XkbVirtualModsMask, xkb);
+      return xkb->ctrls->num_groups;
+}
+
+static gboolean
 update_direction (GdkKeymapX11 *keymap_x11,
 		  gint          group)
 {
-  XkbDescRec *xkb = get_xkb (keymap_x11);
+  XkbDescPtr xkb = get_xkb (keymap_x11);
   Atom group_atom;
+  gboolean had_direction;
+  PangoDirection old_direction;
+      
+  had_direction = keymap_x11->have_direction;
+  old_direction = keymap_x11->current_direction;
 
   group_atom = xkb->names->groups[group];
 
   /* a group change? */
   if (!keymap_x11->have_direction || keymap_x11->current_group_atom != group_atom)
     {
-      gboolean cache_hit = FALSE;
-      DirectionCacheEntry *cache = keymap_x11->group_direction_cache;
-
-      PangoDirection direction = PANGO_DIRECTION_NEUTRAL;
-      gint i;
-
-      if (keymap_x11->have_direction)
-	{
-          /* lookup in cache */
-	  for (i = 0; i < G_N_ELEMENTS (keymap_x11->group_direction_cache); i++)
-	  {
-	    if (cache[i].group_atom == group_atom)
-	      {
-		cache_hit = TRUE;
-		cache[i].serial = keymap_x11->current_cache_serial++; /* freshen */
-		direction = cache[i].direction;
-		group_atom = cache[i].group_atom;
-		break;
-	      }
-	  }
-	}
-      else
-	{
-          /* initialize cache */
-	  for (i = 0; i < G_N_ELEMENTS (keymap_x11->group_direction_cache); i++)
-	    {
-	      cache[i].group_atom = 0;
-	      cache[i].serial = keymap_x11->current_cache_serial;
-	    }
-	  keymap_x11->current_cache_serial++;
-	}
-
-      /* insert in cache */
-      if (!cache_hit)
-	{
-	  gint oldest = 0;
-
-	  direction = get_direction (xkb, group);
-
-	  /* remove the oldest entry */
-	  for (i = 0; i < G_N_ELEMENTS (keymap_x11->group_direction_cache); i++)
-	    {
-	      if (cache[i].serial < cache[oldest].serial)
-	        oldest = i;
-	    }
-	  
-	  cache[oldest].group_atom = group_atom;
-	  cache[oldest].direction = direction;
-	  cache[oldest].serial = keymap_x11->current_cache_serial++;
-	}
-
+      keymap_x11->current_direction = get_direction_from_cache (keymap_x11, xkb, group);
       keymap_x11->current_group_atom = group_atom;
-
       keymap_x11->have_direction = TRUE;
-      keymap_x11->current_direction = direction;
     }
+
+  return !had_direction || old_direction != keymap_x11->current_direction;
+}
+
+static gboolean
+update_lock_state (GdkKeymapX11 *keymap_x11,
+                   gint          locked_mods)
+{
+  gboolean caps_lock_state;
+  
+  caps_lock_state = keymap_x11->caps_lock_state;
+
+  keymap_x11->caps_lock_state = (locked_mods & GDK_LOCK_MASK) != 0;
+  
+  return caps_lock_state != keymap_x11->caps_lock_state;
 }
 
 /* keep this in sync with the XkbSelectEventDetails() call 
@@ -667,16 +708,12 @@ _gdk_keymap_state_changed (GdkDisplay *display,
   if (display_x11->keymap)
     {
       GdkKeymapX11 *keymap_x11 = GDK_KEYMAP_X11 (display_x11->keymap);
-      gboolean had_direction;
-      PangoDirection direction;
       
-      had_direction = keymap_x11->have_direction;
-      direction = keymap_x11->current_direction;
-      
-      update_direction (keymap_x11, xkb_event->state.locked_group);
-      
-      if (!had_direction || direction != keymap_x11->current_direction)
-	g_signal_emit_by_name (keymap_x11, "direction_changed");      
+      if (update_direction (keymap_x11, XkbStateGroup (&xkb_event->state)))
+	g_signal_emit_by_name (keymap_x11, "direction-changed");      
+
+      if (update_lock_state (keymap_x11, xkb_event->state.locked_mods))
+	g_signal_emit_by_name (keymap_x11, "state-changed");      
     }
 }
 
@@ -693,6 +730,16 @@ _gdk_keymap_keys_changed (GdkDisplay *display)
     g_signal_emit_by_name (display_x11->keymap, "keys_changed", 0);
 }
 
+/** 
+ * gdk_keymap_get_direction:
+ * @keymap: a #GdkKeymap or %NULL to use the default keymap
+ *
+ * Returns the direction of effective layout of the keymap.
+ *
+ * Returns: %PANGO_DIRECTION_LTR or %PANGO_DIRECTION_RTL 
+ *   if it can determine the direction. %PANGO_DIRECTION_NEUTRAL 
+ *   otherwise.
+ **/
 PangoDirection
 gdk_keymap_get_direction (GdkKeymap *keymap)
 {
@@ -710,7 +757,7 @@ gdk_keymap_get_direction (GdkKeymap *keymap)
 
 	  XkbGetState (GDK_DISPLAY_XDISPLAY (display), XkbUseCoreKbd, 
 		       &state_rec);
-	  update_direction (keymap_x11, XkbGroupLock (&state_rec));
+	  update_direction (keymap_x11, XkbStateGroup (&state_rec));
 	}
   
       return keymap_x11->current_direction;
@@ -720,6 +767,71 @@ gdk_keymap_get_direction (GdkKeymap *keymap)
     return PANGO_DIRECTION_NEUTRAL;
 }
 
+/** 
+ * gdk_keymap_have_bidi_layouts:
+ * @keymap: a #GdkKeymap or %NULL to use the default keymap
+ *
+ * Determines if keyboard layouts for both right-to-left and left-to-right
+ * languages are in use.
+ *
+ * Returns: %TRUE if there are layouts in both directions, %FALSE otherwise
+ *
+ * Since: 2.12
+ **/
+gboolean
+gdk_keymap_have_bidi_layouts (GdkKeymap *keymap)
+{
+  keymap = GET_EFFECTIVE_KEYMAP (keymap);
+
+#if HAVE_XKB
+  if (KEYMAP_USE_XKB (keymap))
+    {
+      GdkKeymapX11 *keymap_x11 = GDK_KEYMAP_X11 (keymap);
+      XkbDescPtr xkb = get_xkb (keymap_x11);
+      int num_groups = get_num_groups (keymap, xkb);
+
+      int i;
+      gboolean have_ltr_keyboard = FALSE;
+      gboolean have_rtl_keyboard = FALSE;
+
+      for (i = 0; i < num_groups; i++)
+      {
+	if (get_direction_from_cache (keymap_x11, xkb, i) == PANGO_DIRECTION_RTL)
+	  have_rtl_keyboard = TRUE;
+	else
+	  have_ltr_keyboard = TRUE;
+      }
+
+      return have_ltr_keyboard && have_rtl_keyboard;
+    }
+  else
+#endif /* HAVE_XKB */
+    return FALSE;
+}
+
+/**
+ * gdk_keymap_get_caps_lock_state:
+ * @keymap: a #GdkKeymap
+ *
+ * Returns whether the Caps Lock modifer is locked. 
+ *
+ * Returns: %TRUE if Caps Lock is on
+ *
+ * Since: 2.16
+ */
+gboolean
+gdk_keymap_get_caps_lock_state (GdkKeymap *keymap)
+{
+  GdkKeymapX11 *keymap_x11;
+  
+  keymap = GET_EFFECTIVE_KEYMAP (keymap);
+  
+  keymap_x11 = GDK_KEYMAP_X11 (keymap);
+  
+  return keymap_x11->caps_lock_state;
+}
+
+
 /**
  * gdk_keymap_get_entries_for_keyval:
  * @keymap: a #GdkKeymap, or %NULL to use the default keymap
@@ -1086,8 +1198,8 @@ MyEnhancedXkbTranslateKeyCode(register XkbDescPtr     xkb,
                               register unsigned int   mods,
                               unsigned int *          mods_rtrn,
                               KeySym *                keysym_rtrn,
-                              unsigned int *          group_rtrn,
-                              unsigned int *          level_rtrn)
+                              int *                   group_rtrn,
+                              int *                   level_rtrn)
 {
     XkbKeyTypeRec *type;
     int col,nKeyGroups;
@@ -1212,8 +1324,8 @@ translate_keysym (GdkKeymapX11   *keymap_x11,
 		  guint           hardware_keycode,
 		  gint            group,
 		  GdkModifierType state,
-		  guint          *effective_group,
-		  guint          *effective_level)
+		  gint           *effective_group,
+		  gint           *effective_level)
 {
   const KeySym *map = get_keymap (keymap_x11);
   const KeySym *syms = map + (hardware_keycode - keymap_x11->min_keycode) * keymap_x11->keysyms_per_keycode;
@@ -1589,7 +1701,7 @@ _gdk_keymap_key_is_modifier (GdkKeymap *keymap,
     {
       XkbDescRec *xkb = get_xkb (keymap_x11);
       
-      if (xkb->map->modmap[keycode] != 0)
+      if (xkb->map->modmap && xkb->map->modmap[keycode] != 0)
 	return TRUE;
     }
   else
