@@ -17,7 +17,7 @@
  * Boston, MA 02111-1307, USA.
  */
 
-#include <config.h>
+#include "config.h"
 #include "gtkcellrenderer.h"
 #include "gtkintl.h"
 #include "gtkmarshalers.h"
@@ -47,7 +47,7 @@ struct _GtkCellRendererPrivate
 
 
 enum {
-  PROP_ZERO,
+  PROP_0,
   PROP_MODE,
   PROP_VISIBLE,
   PROP_SENSITIVE,
@@ -61,7 +61,8 @@ enum {
   PROP_IS_EXPANDED,
   PROP_CELL_BACKGROUND,
   PROP_CELL_BACKGROUND_GDK,
-  PROP_CELL_BACKGROUND_SET
+  PROP_CELL_BACKGROUND_SET,
+  PROP_EDITING
 };
 
 /* Signal IDs */
@@ -139,8 +140,7 @@ gtk_cell_renderer_class_init (GtkCellRendererClass *class)
    * continue to use the same kind of widget for editing in future
    * releases, therefore you should check the type of @editable
    * before doing any specific setup, as in the following example:
-   *
-   * <informalexample><programlisting>
+   * |[
    * static void
    * text_editing_started (GtkCellRenderer *cell,
    *                       GtkCellEditable *editable,
@@ -150,13 +150,13 @@ gtk_cell_renderer_class_init (GtkCellRendererClass *class)
    *   if (GTK_IS_ENTRY (editable)) 
    *     {
    *       GtkEntry *entry = GTK_ENTRY (editable);
-   *       <!-- -->
-   *       /<!-- -->* ... create a GtkEntryCompletion *<!-- -->/
-   *       <!-- -->
+   *       
+   *       /&ast; ... create a GtkEntryCompletion &ast;/
+   *       
    *       gtk_entry_set_completion (entry, completion);
    *     }
    * }
-   * </programlisting></informalexample>
+   * ]|
    *
    * Since: 2.6
    */
@@ -288,6 +288,14 @@ gtk_cell_renderer_class_init (GtkCellRendererClass *class)
 						       GDK_TYPE_COLOR,
 						       GTK_PARAM_READWRITE));
 
+  g_object_class_install_property (object_class,
+				   PROP_EDITING,
+				   g_param_spec_boolean ("editing",
+							 P_("Editing"),
+							 P_("Whether the cell renderer is currently in editing mode"),
+							 FALSE,
+							 GTK_PARAM_READABLE));
+
 
 #define ADD_SET_PROP(propname, propval, nick, blurb) g_object_class_install_property (object_class, propval, g_param_spec_boolean (propname, nick, blurb, FALSE, GTK_PARAM_READWRITE))
 
@@ -318,6 +326,9 @@ gtk_cell_renderer_get_property (GObject     *object,
     case PROP_SENSITIVE:
       g_value_set_boolean (value, cell->sensitive);
       break;
+    case PROP_EDITING:
+      g_value_set_boolean (value, cell->editing);
+      break;
     case PROP_XALIGN:
       g_value_set_float (value, cell->xalign);
       break;
@@ -383,6 +394,9 @@ gtk_cell_renderer_set_property (GObject      *object,
     case PROP_SENSITIVE:
       cell->sensitive = g_value_get_boolean (value);
       break;
+    case PROP_EDITING:
+      cell->editing = g_value_get_boolean (value);
+      break;
     case PROP_XALIGN:
       cell->xalign = g_value_get_float (value);
       break;
@@ -480,13 +494,13 @@ set_cell_bg_color (GtkCellRenderer *cell,
  * in @x_offset and @y_offset are inclusive of the xpad and ypad properties.
  **/
 void
-gtk_cell_renderer_get_size (GtkCellRenderer *cell,
-			    GtkWidget       *widget,
-			    GdkRectangle    *cell_area,
-			    gint            *x_offset,
-			    gint            *y_offset,
-			    gint            *width,
-			    gint            *height)
+gtk_cell_renderer_get_size (GtkCellRenderer    *cell,
+			    GtkWidget          *widget,
+			    const GdkRectangle *cell_area,
+			    gint               *x_offset,
+			    gint               *y_offset,
+			    gint               *width,
+			    gint               *height)
 {
   gint *real_width = width;
   gint *real_height = height;
@@ -507,7 +521,7 @@ gtk_cell_renderer_get_size (GtkCellRenderer *cell,
 
   GTK_CELL_RENDERER_GET_CLASS (cell)->get_size (cell,
 						widget,
-						cell_area,
+						(GdkRectangle *) cell_area,
 						x_offset,
 						y_offset,
 						real_width,
@@ -534,13 +548,13 @@ gtk_cell_renderer_get_size (GtkCellRenderer *cell,
  * @window.  @expose_area is a clip rectangle.
  **/
 void
-gtk_cell_renderer_render (GtkCellRenderer     *cell,
-			  GdkWindow           *window,
-			  GtkWidget           *widget,
-			  GdkRectangle        *background_area,
-			  GdkRectangle        *cell_area,
-			  GdkRectangle        *expose_area,
-			  GtkCellRendererState flags)
+gtk_cell_renderer_render (GtkCellRenderer      *cell,
+			  GdkWindow            *window,
+			  GtkWidget            *widget,
+			  const GdkRectangle   *background_area,
+			  const GdkRectangle   *cell_area,
+			  const GdkRectangle   *expose_area,
+			  GtkCellRendererState  flags)
 {
   gboolean selected = FALSE;
   GtkCellRendererPrivate *priv = GTK_CELL_RENDERER_GET_PRIVATE (cell);
@@ -564,9 +578,9 @@ gtk_cell_renderer_render (GtkCellRenderer     *cell,
   GTK_CELL_RENDERER_GET_CLASS (cell)->render (cell,
 					      window,
 					      widget,
-					      background_area,
-					      cell_area,
-					      expose_area,
+					      (GdkRectangle *) background_area,
+					      (GdkRectangle *) cell_area,
+					      (GdkRectangle *) expose_area,
 					      flags);
 }
 
@@ -592,8 +606,8 @@ gtk_cell_renderer_activate (GtkCellRenderer      *cell,
 			    GdkEvent             *event,
 			    GtkWidget            *widget,
 			    const gchar          *path,
-			    GdkRectangle         *background_area,
-			    GdkRectangle         *cell_area,
+			    const GdkRectangle   *background_area,
+			    const GdkRectangle   *cell_area,
 			    GtkCellRendererState  flags)
 {
   g_return_val_if_fail (GTK_IS_CELL_RENDERER (cell), FALSE);
@@ -608,8 +622,8 @@ gtk_cell_renderer_activate (GtkCellRenderer      *cell,
 						       event,
 						       widget,
 						       path,
-						       background_area,
-						       cell_area,
+						       (GdkRectangle *) background_area,
+						       (GdkRectangle *) cell_area,
 						       flags);
 }
 
@@ -633,8 +647,8 @@ gtk_cell_renderer_start_editing (GtkCellRenderer      *cell,
 				 GdkEvent             *event,
 				 GtkWidget            *widget,
 				 const gchar          *path,
-				 GdkRectangle         *background_area,
-				 GdkRectangle         *cell_area,
+				 const GdkRectangle   *background_area,
+				 const GdkRectangle   *cell_area,
 				 GtkCellRendererState  flags)
 
 {
@@ -652,8 +666,8 @@ gtk_cell_renderer_start_editing (GtkCellRenderer      *cell,
 								event,
 								widget,
 								path,
-								background_area,
-								cell_area,
+								(GdkRectangle *) background_area,
+								(GdkRectangle *) cell_area,
 								flags);
 
   g_signal_emit (cell, 
@@ -670,7 +684,7 @@ gtk_cell_renderer_start_editing (GtkCellRenderer      *cell,
  * @cell: A #GtkCellRenderer
  * @width: the width of the cell renderer, or -1
  * @height: the height of the cell renderer, or -1
- * 
+ *
  * Sets the renderer size to be explicit, independent of the properties set.
  **/
 void
@@ -704,9 +718,9 @@ gtk_cell_renderer_set_fixed_size (GtkCellRenderer *cell,
 /**
  * gtk_cell_renderer_get_fixed_size:
  * @cell: A #GtkCellRenderer
- * @width: location to fill in with the fixed width of the widget, or %NULL
- * @height: location to fill in with the fixed height of the widget, or %NULL
- * 
+ * @width: location to fill in with the fixed width of the cell, or %NULL
+ * @height: location to fill in with the fixed height of the cell, or %NULL
+ *
  * Fills in @width and @height with the appropriate size of @cell.
  **/
 void
@@ -717,9 +731,212 @@ gtk_cell_renderer_get_fixed_size (GtkCellRenderer *cell,
   g_return_if_fail (GTK_IS_CELL_RENDERER (cell));
 
   if (width)
-    (* width) = cell->width;
+    *width = cell->width;
   if (height)
-    (* height) = cell->height;
+    *height = cell->height;
+}
+
+/**
+ * gtk_cell_renderer_set_alignment:
+ * @cell: A #GtkCellRenderer
+ * @xalign: the x alignment of the cell renderer
+ * @yalign: the y alignment of the cell renderer
+ *
+ * Sets the renderer's alignment within its available space.
+ *
+ * Since: 2.18
+ **/
+void
+gtk_cell_renderer_set_alignment (GtkCellRenderer *cell,
+                                 gfloat           xalign,
+                                 gfloat           yalign)
+{
+  g_return_if_fail (GTK_IS_CELL_RENDERER (cell));
+  g_return_if_fail (xalign >= 0.0 && xalign <= 1.0);
+  g_return_if_fail (yalign >= 0.0 && yalign <= 1.0);
+
+  if ((xalign != cell->xalign) || (yalign != cell->yalign))
+    {
+      g_object_freeze_notify (G_OBJECT (cell));
+
+      if (xalign != cell->xalign)
+        {
+          cell->xalign = xalign;
+          g_object_notify (G_OBJECT (cell), "xalign");
+        }
+
+      if (yalign != cell->yalign)
+        {
+          cell->yalign = yalign;
+          g_object_notify (G_OBJECT (cell), "yalign");
+        }
+
+      g_object_thaw_notify (G_OBJECT (cell));
+    }
+}
+
+/**
+ * gtk_cell_renderer_get_alignment:
+ * @cell: A #GtkCellRenderer
+ * @xalign: location to fill in with the x alignment of the cell, or %NULL
+ * @yalign: location to fill in with the y alignment of the cell, or %NULL
+ *
+ * Fills in @xalign and @yalign with the appropriate values of @cell.
+ *
+ * Since: 2.18
+ **/
+void
+gtk_cell_renderer_get_alignment (GtkCellRenderer *cell,
+                                 gfloat          *xalign,
+                                 gfloat          *yalign)
+{
+  g_return_if_fail (GTK_IS_CELL_RENDERER (cell));
+
+  if (xalign)
+    *xalign = cell->xalign;
+  if (yalign)
+    *yalign = cell->yalign;
+}
+
+/**
+ * gtk_cell_renderer_set_padding:
+ * @cell: A #GtkCellRenderer
+ * @xpad: the x padding of the cell renderer
+ * @ypad: the y padding of the cell renderer
+ *
+ * Sets the renderer's padding.
+ *
+ * Since: 2.18
+ **/
+void
+gtk_cell_renderer_set_padding (GtkCellRenderer *cell,
+                               gint             xpad,
+                               gint             ypad)
+{
+  g_return_if_fail (GTK_IS_CELL_RENDERER (cell));
+  g_return_if_fail (xpad >= 0 && xpad >= 0);
+
+  if ((xpad != cell->xpad) || (ypad != cell->ypad))
+    {
+      g_object_freeze_notify (G_OBJECT (cell));
+
+      if (xpad != cell->xpad)
+        {
+          cell->xpad = xpad;
+          g_object_notify (G_OBJECT (cell), "xpad");
+        }
+
+      if (ypad != cell->ypad)
+        {
+          cell->ypad = ypad;
+          g_object_notify (G_OBJECT (cell), "ypad");
+        }
+
+      g_object_thaw_notify (G_OBJECT (cell));
+    }
+}
+
+/**
+ * gtk_cell_renderer_get_padding:
+ * @cell: A #GtkCellRenderer
+ * @xpad: location to fill in with the x padding of the cell, or %NULL
+ * @ypad: location to fill in with the y padding of the cell, or %NULL
+ *
+ * Fills in @xpad and @ypad with the appropriate values of @cell.
+ *
+ * Since: 2.18
+ **/
+void
+gtk_cell_renderer_get_padding (GtkCellRenderer *cell,
+                               gint            *xpad,
+                               gint            *ypad)
+{
+  g_return_if_fail (GTK_IS_CELL_RENDERER (cell));
+
+  if (xpad)
+    *xpad = cell->xpad;
+  if (ypad)
+    *ypad = cell->ypad;
+}
+
+/**
+ * gtk_cell_renderer_set_visible:
+ * @cell: A #GtkCellRenderer
+ * @visible: the visibility of the cell
+ *
+ * Sets the cell renderer's visibility.
+ *
+ * Since: 2.18
+ **/
+void
+gtk_cell_renderer_set_visible (GtkCellRenderer *cell,
+                               gboolean         visible)
+{
+  g_return_if_fail (GTK_IS_CELL_RENDERER (cell));
+
+  if (cell->visible != visible)
+    {
+      cell->visible = visible ? TRUE : FALSE;
+      g_object_notify (G_OBJECT (cell), "visible");
+    }
+}
+
+/**
+ * gtk_cell_renderer_get_visible:
+ * @cell: A #GtkCellRenderer
+ *
+ * Returns the cell renderer's visibility.
+ *
+ * Returns: %TRUE if the cell renderer is visible
+ *
+ * Since: 2.18
+ */
+gboolean
+gtk_cell_renderer_get_visible (GtkCellRenderer *cell)
+{
+  g_return_val_if_fail (GTK_IS_CELL_RENDERER (cell), FALSE);
+
+  return cell->visible;
+}
+
+/**
+ * gtk_cell_renderer_set_sensitive:
+ * @cell: A #GtkCellRenderer
+ * @sensitive: the sensitivity of the cell
+ *
+ * Sets the cell renderer's sensitivity.
+ *
+ * Since: 2.18
+ **/
+void
+gtk_cell_renderer_set_sensitive (GtkCellRenderer *cell,
+                                 gboolean         sensitive)
+{
+  g_return_if_fail (GTK_IS_CELL_RENDERER (cell));
+
+  if (cell->sensitive != sensitive)
+    {
+      cell->sensitive = sensitive ? TRUE : FALSE;
+      g_object_notify (G_OBJECT (cell), "sensitive");
+    }
+}
+
+/**
+ * gtk_cell_renderer_get_sensitive:
+ * @cell: A #GtkCellRenderer
+ *
+ * Returns the cell renderer's sensitivity.
+ *
+ * Returns: %TRUE if the cell renderer is sensitive
+ *
+ * Since: 2.18
+ */
+gboolean
+gtk_cell_renderer_get_sensitive (GtkCellRenderer *cell)
+{
+  g_return_val_if_fail (GTK_IS_CELL_RENDERER (cell), FALSE);
+
+  return cell->sensitive;
 }
 
 /**
