@@ -21,18 +21,20 @@
  * Modified by the GTK+ Team and others 1997-2000.  See the AUTHORS
  * file for a list of people on the GTK+ Team.  See the ChangeLog
  * files for a list of changes.  These files are distributed with
- * GTK+ at ftp://ftp.gtk.org/pub/gtk/. 
+ * GTK+ at ftp://ftp.gtk.org/pub/gtk/.
  */
 
+#if defined(GTK_DISABLE_SINGLE_INCLUDES) && !defined (__GTK_H_INSIDE__) && !defined (GTK_COMPILATION)
+#error "Only <gtk/gtk.h> can be included directly."
+#endif
+
 #ifndef __GTK_WINDOW_H__
 #define __GTK_WINDOW_H__
 
 
-#include <gdk/gdk.h>
 #include <gtk/gtkaccelgroup.h>
 #include <gtk/gtkbin.h>
-#include <gtk/gtkenums.h>
-#include <gtk/gtkwidget.h>
+
 
 G_BEGIN_DECLS
 
@@ -53,63 +55,63 @@ struct _GtkWindow
 {
   GtkBin bin;
 
-  gchar *title;
-  gchar *wmclass_name;
-  gchar *wmclass_class;
-  gchar *wm_role;
+  gchar *GSEAL (title);
+  gchar *GSEAL (wmclass_name);
+  gchar *GSEAL (wmclass_class);
+  gchar *GSEAL (wm_role);
 
-  GtkWidget *focus_widget;
-  GtkWidget *default_widget;
-  GtkWindow *transient_parent;
-  GtkWindowGeometryInfo *geometry_info;
-  GdkWindow *frame;
-  GtkWindowGroup *group;
+  GtkWidget *GSEAL (focus_widget);
+  GtkWidget *GSEAL (default_widget);
+  GtkWindow *GSEAL (transient_parent);
+  GtkWindowGeometryInfo *GSEAL (geometry_info);
+  GdkWindow *GSEAL (frame);
+  GtkWindowGroup *GSEAL (group);
 
-  guint16 configure_request_count;
-  guint allow_shrink : 1;
-  guint allow_grow : 1;
-  guint configure_notify_received : 1;
+  guint16 GSEAL (configure_request_count);
+  guint GSEAL (allow_shrink) : 1;
+  guint GSEAL (allow_grow) : 1;
+  guint GSEAL (configure_notify_received) : 1;
   /* The following flags are initially TRUE (before a window is mapped).
    * They cause us to compute a configure request that involves
    * default-only parameters. Once mapped, we set them to FALSE.
    * Then we set them to TRUE again on unmap (for position)
    * and on unrealize (for size).
    */
-  guint need_default_position : 1;
-  guint need_default_size : 1;
-  guint position : 3;
-  guint type : 4; /* GtkWindowType */ 
-  guint has_user_ref_count : 1;
-  guint has_focus : 1;
+  guint GSEAL (need_default_position) : 1;
+  guint GSEAL (need_default_size) : 1;
+  guint GSEAL (position) : 3;
+  guint GSEAL (type) : 4; /* GtkWindowType */ 
+  guint GSEAL (has_user_ref_count) : 1;
+  guint GSEAL (has_focus) : 1;
 
-  guint modal : 1;
-  guint destroy_with_parent : 1;
+  guint GSEAL (modal) : 1;
+  guint GSEAL (destroy_with_parent) : 1;
   
-  guint has_frame : 1;
+  guint GSEAL (has_frame) : 1;
 
   /* gtk_window_iconify() called before realization */
-  guint iconify_initially : 1;
-  guint stick_initially : 1;
-  guint maximize_initially : 1;
-  guint decorated : 1;
+  guint GSEAL (iconify_initially) : 1;
+  guint GSEAL (stick_initially) : 1;
+  guint GSEAL (maximize_initially) : 1;
+  guint GSEAL (decorated) : 1;
   
-  guint type_hint : 3; /* GdkWindowTypeHint if the hint is one of the original eight. If not, then
-			* it contains GDK_WINDOW_TYPE_HINT_NORMAL
-			*/
-  guint gravity : 5; /* GdkGravity */
+  guint GSEAL (type_hint) : 3; /* GdkWindowTypeHint if the hint is one of the original eight. If not, then
+				* it contains GDK_WINDOW_TYPE_HINT_NORMAL
+				*/
+  guint GSEAL (gravity) : 5; /* GdkGravity */
 
-  guint is_active : 1;
-  guint has_toplevel_focus : 1;
+  guint GSEAL (is_active) : 1;
+  guint GSEAL (has_toplevel_focus) : 1;
   
-  guint frame_left;
-  guint frame_top;
-  guint frame_right;
-  guint frame_bottom;
+  guint GSEAL (frame_left);
+  guint GSEAL (frame_top);
+  guint GSEAL (frame_right);
+  guint GSEAL (frame_bottom);
 
-  guint keys_changed_handler;
+  guint GSEAL (keys_changed_handler);
   
-  GdkModifierType mnemonic_modifier;
-  GdkScreen      *screen;
+  GdkModifierType GSEAL (mnemonic_modifier);
+  GdkScreen      *GSEAL (screen);
 };
 
 struct _GtkWindowClass
@@ -153,7 +155,7 @@ struct _GtkWindowGroup
 {
   GObject parent_instance;
 
-  GSList *grabs;
+  GSList *GSEAL (grabs);
 };
 
 struct _GtkWindowGroupClass
@@ -198,6 +200,7 @@ void       gtk_window_set_focus                (GtkWindow           *window,
 GtkWidget *gtk_window_get_focus                (GtkWindow           *window);
 void       gtk_window_set_default              (GtkWindow           *window,
 						GtkWidget           *default_widget);
+GtkWidget *gtk_window_get_default_widget       (GtkWindow           *window);
 gboolean   gtk_window_activate_default	       (GtkWindow           *window);
 
 void       gtk_window_set_transient_for        (GtkWindow           *window, 
@@ -289,6 +292,8 @@ void       gtk_window_set_default_icon_list        (GList      *list);
 GList*     gtk_window_get_default_icon_list        (void);
 void       gtk_window_set_default_icon             (GdkPixbuf  *icon);
 void       gtk_window_set_default_icon_name        (const gchar *name);
+G_CONST_RETURN
+gchar     *gtk_window_get_default_icon_name        (void);
 gboolean   gtk_window_set_default_icon_from_file   (const gchar *filename,
 						    GError     **err);
 
@@ -389,14 +394,16 @@ void             gtk_window_group_add_window    (GtkWindowGroup     *window_grou
 						 GtkWindow          *window);
 void             gtk_window_group_remove_window (GtkWindowGroup     *window_group,
 					         GtkWindow          *window);
+GList *          gtk_window_group_list_windows  (GtkWindowGroup     *window_group);
+
 
 /* --- internal functions --- */
 void            _gtk_window_internal_set_focus (GtkWindow *window,
 						GtkWidget *focus);
-void            gtk_window_remove_embedded_xid (GtkWindow *window,
-						guint      xid);
-void            gtk_window_add_embedded_xid    (GtkWindow *window,
-						guint      xid);
+void            gtk_window_remove_embedded_xid (GtkWindow       *window,
+						GdkNativeWindow  xid);
+void            gtk_window_add_embedded_xid    (GtkWindow       *window,
+						GdkNativeWindow  xid);
 void            _gtk_window_reposition         (GtkWindow *window,
 						gint       x,
 						gint       y);
@@ -415,6 +422,9 @@ void            _gtk_window_unset_focus_and_default (GtkWindow *window,
 void            _gtk_window_set_is_active          (GtkWindow *window,
 						    gboolean   is_active);
 
+void            _gtk_window_set_is_toplevel        (GtkWindow *window,
+						    gboolean   is_toplevel);
+
 typedef void (*GtkWindowKeysForeachFunc) (GtkWindow      *window,
 					  guint           keyval,
 					  GdkModifierType modifiers,
